/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qfile.h>
#include <qtextstream.h>
#include <qiodevice.h>
#include <qstringlist.h>

#include <settings.h>
#include <bibtexfile.h>
#include <bibtexentry.h>
#include <bibtexelement.h>
#include <bibtexstring.h>
#include <bibtexcomment.h>
#include <bibtexpreamble.h>
#include <bibtexfileparser.h>
#include <bibtexfileexporter.h>

BibTeXFile::BibTeXFile( QObject *parent, const char *name ) : QObject( parent, name ), parser( NULL ), exporter( NULL )
{
    //TODO
}

BibTeXFile::~BibTeXFile()
{
    clearElements();
}

bool BibTeXFile::load( const QString& filename )
{
    bool result = false;

    QFile input( filename );
    if ( input.exists() && input.open( IO_ReadOnly ) )
    {
        result = load( &input );
        input.close();
    }

    return result;
}

bool BibTeXFile::load( QIODevice *iodevice )
{
    QTextStream stream( iodevice );
    return load( stream );
}

bool BibTeXFile::load( QTextStream &stream )
{
    bool result = true;
    clearElements();

    parser = new BibTeXFileParser( &stream );
    connect( parser, SIGNAL( progress( int, int ) ), this , SLOT( slotProgress( int, int ) ) );
    BibTeXElement *element;
    while ( ( element = parser->nextElement() ) )
        elements.append( element );

    disconnect( parser, SIGNAL( progress( int, int ) ), this , SLOT( slotProgress( int, int ) ) );
    delete( parser );
    parser = NULL;

    return result;
}

bool BibTeXFile::save( const QString& filename, BibTeXFile::Format format, BibTeXFile::Encoding encoding )
{
    bool result = false;

    QFile output( filename );
    if ( output.open( IO_WriteOnly ) )
    {
        result = save( &output , format, encoding );
        output.close();
    }

    return result;
}

bool BibTeXFile::save( QIODevice *iodevice, BibTeXFile::Format format, BibTeXFile::Encoding encoding )
{
    exporter = new BibTeXFileExporter( this );
    connect( exporter, SIGNAL( progress( int, int ) ), this , SLOT( slotProgress( int, int ) ) );
    bool result = exporter->save( iodevice, format, encoding );
    disconnect( exporter, SIGNAL( progress( int, int ) ), this , SLOT( slotProgress( int, int ) ) );
    delete exporter;
    exporter = NULL;

    return result;
}

unsigned int BibTeXFile::count ()
{
    return elements.count();
}

BibTeXElement* BibTeXFile::at( const unsigned int index )
{
    return * ( elements.at( index ) );
}

void BibTeXFile::appendElement( BibTeXElement *element )
{
    elements.append( element );
}

void BibTeXFile::deleteElement( BibTeXElement *element )
{
    for ( ElementList::iterator it = elements.begin(); it != elements.end(); it++ )
        if ( *it == element )
        {
            elements.remove( it );
            break;
        }

    delete element;
}

void BibTeXFile::insertText( QString &text )
{
    BibTeXFile * bibtexfile = new BibTeXFile();
    QTextStream textStream( text, IO_ReadOnly );
    bibtexfile->load( textStream );

    for ( unsigned int i = 0; i < bibtexfile->count(); i++ )
    {
        BibTeXElement* originalElement = bibtexfile->at( i );

        BibTeXEntry* originalEntry = dynamic_cast<BibTeXEntry*>( originalElement );
        if ( originalEntry != NULL )
            appendElement( new BibTeXEntry( originalEntry ) );
        else
        {
            BibTeXComment* originalComment = dynamic_cast<BibTeXComment*>( originalElement );
            if ( originalComment != NULL )
                appendElement( new BibTeXComment( originalComment ) );
            else
            {
                BibTeXString* originalString = dynamic_cast<BibTeXString*>( originalElement );
                if ( originalString != NULL )
                    appendElement ( new BibTeXString( originalString ) );
                else
                {
                    BibTeXPreamble* originalPreamble = dynamic_cast<BibTeXPreamble*>( originalElement );
                    if ( originalPreamble != NULL )
                        appendElement( new BibTeXPreamble( originalPreamble ) );
                }
            }
        }
    }

    delete bibtexfile;
}

void BibTeXFile::cancel()
{
    qDebug( "cancel" );
    if ( exporter != NULL )
    {
        exporter->cancel();
    }
    if ( parser != NULL )
    {
        parser->cancel();
    }
}

void BibTeXFile::clearElements()
{
    for ( ElementList::iterator it = elements.begin(); it != elements.end(); it++ )
    {
        delete( *it );
    }
    elements.clear();
}

void BibTeXFile::slotProgress( int current, int total )
{
    emit progress( current, total );
}

#include "bibtexfile.moc"

