/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include <qapplication.h>

#include <bibtexelement.h>
#include <bibtexstring.h>
#include <bibtexcomment.h>
#include <bibtexpreamble.h>
#include <bibtexentry.h>
#include "bibtexfileparser.h"

const QString ExtraAlphaNumChars = QString( "?'`-_:.+/$\\\"" );

BibTeXFileParser::BibTeXFileParser( QTextStream *inputStream ) : QObject(), cancelFlag( false ), tokenChar( '\0' )
{
    stream = inputStream;
    token = readToken( );
    emit progress( 0, stream->device() ->size() );
}

QString BibTeXFileParser::readChars( )
{
    QString buffer = "";
    QChar c, old_c = '\0';
    StringTerminator stringTerminator;
    int curlyBracketCounter = 0;
    int parenthesisCounter = 0;
    int quotationCounter = 0;
    int whitespaceCounter = 0;

    stream->skipWhiteSpace();

    *stream >> c;
    switch ( c.latin1() )
    {
    case '"': stringTerminator = stQuotationMark; break;
    case '(': stringTerminator = stParenthesis; break;
    case '{': stringTerminator = stCurlyBracket; break;
    default: stringTerminator = stWhiteSpace; buffer += c; break;
    }

    bool done = FALSE;
    do
    {
        old_c = c;
        *stream >> c;

        switch ( c.latin1() )
        {
        case '{':
            buffer += c;
            if ( stringTerminator == stCurlyBracket && old_c != '\\' )
                curlyBracketCounter++;
            break;
        case '}':
            if ( stringTerminator == stCurlyBracket && old_c != '\\' )
                curlyBracketCounter--;
            if ( curlyBracketCounter >= 0 )
                buffer += c;
            break;
        case '(':
            buffer += c;
            if ( stringTerminator == stParenthesis && old_c != '\\' )
                parenthesisCounter++;
            break;
        case ')':
            if ( stringTerminator == stParenthesis && old_c != '\\' )
                parenthesisCounter--;
            if ( parenthesisCounter >= 0 )
                buffer += c;
            break;
        case '"':
            if ( stringTerminator == stQuotationMark && old_c != '\\' )
                quotationCounter = 1 - quotationCounter;
            else
                buffer += c;
            break;
        default:
            if ( stringTerminator == stWhiteSpace && !c.isLetterOrNumber() && ( ExtraAlphaNumChars.find( c ) == -1 ) )
            {
                done = TRUE;
                break;
            }

            if ( c.latin1() < 16 )
                c = ' ';
            if ( !c.isSpace() || !old_c.isSpace() )
                buffer += c;
            if ( c.isSpace() )
                whitespaceCounter++;
        }

        done |= ( stringTerminator == stCurlyBracket && curlyBracketCounter == -1 ) || ( stringTerminator == stParenthesis && parenthesisCounter == -1 ) || ( stringTerminator == stQuotationMark && quotationCounter == 1 ) || stream->atEnd();
    }
    while ( ! done );

    if ( stringTerminator == stWhiteSpace )
        tokenChar = c;

    //     qDebug( "readChars = %s", buffer.latin1() );

    return buffer;
}

BibTeXString* BibTeXFileParser::readString( )
{
    BibTeXString * result = NULL;

    if ( ( readToken( ) == tBraketOpen ) && ( readToken( ) == tAlphaNumText ) )
    {
        QString id = tokenBuffer;
        if ( readToken( ) == tAssign )
        {
            QString value = readChars( );

            result = new BibTeXString( id, value );
        }
        while ( readToken( ) != tBraketClose )
            ;
    }

    return result;
}

BibTeXEntry* BibTeXFileParser::readEntry( QString &entryType )
{
    BibTeXEntry * result = NULL;

    if ( ( readToken( ) == tBraketOpen ) && ( readToken( ) == tAlphaNumText ) )
    {
        result = new BibTeXEntry( BibTeXEntry::entryTypeFromString( entryType ), tokenBuffer );

        QString fieldName;
        Token token = readToken( );
        while ( token != tBraketClose )
        {
            if ( token == tAlphaNumText )
            {
                fieldName = tokenBuffer;
            }
            else if ( token == tAssign )
            {
                QString value = readChars( );
                BibTeXEntryField* field = new BibTeXEntryField( fieldName, value );
                result->addField( field );
            }
            else if ( token == tEOF )
            {
                delete ( result );
                result = NULL;
                break;
            }
            token = readToken( );
        }

    }

    return result;
}

BibTeXFileParser::Token BibTeXFileParser::readToken( )
{
    if ( ( tokenChar == '\0' ) || ( tokenChar.isSpace() ) )
    {
        stream->skipWhiteSpace();
        *stream >> tokenChar;
    }

    //     qDebug( "tokenChar = %c", tokenChar.latin1() );

    if ( tokenChar == '@' )
    {
        *stream >> tokenChar;
        tokenBuffer = "";
        while ( !stream->atEnd() && tokenChar.isLetter() )
        {
            tokenBuffer += tokenChar.lower();
            *stream >> tokenChar;
        }

        if ( tokenBuffer == "comment" )
        {
            tokenBuffer = "";
            return tComment;
        }
        else if ( tokenBuffer == "preamble" )
        {
            tokenBuffer = "";
            return tPreamble;
        }
        else if ( tokenBuffer == "string" )
        {
            tokenBuffer = "";
            return tString;
        }
        else
            return tEntry;
    }
    else if ( ( tokenChar == '{' ) || ( tokenChar == '(' ) )
    {
        tokenChar = '\0';
        return tBraketOpen;
    }
    else if ( ( tokenChar == '}' ) || ( tokenChar == ')' ) )
    {
        tokenChar = '\0';
        return tBraketClose;
    }
    else if ( tokenChar == ',' )
    {
        tokenChar = '\0';
        return tComma;
    }
    else if ( tokenChar == ';' )
    {
        tokenChar = '\0';
        return tSemicolon;
    }
    else if ( tokenChar == '=' )
    {
        tokenChar = '\0';
        return tAssign;
    }
    else if ( tokenChar == '%' )
    {
        tokenBuffer = "%" + stream->readLine();
        *stream >> tokenChar;
        while ( !stream->atEnd() && tokenChar == '%' )
        {
            tokenBuffer += "\n%";
            tokenBuffer += stream->readLine();
            *stream >> tokenChar;
        }

        return tComment;
    }
    else if ( tokenChar == '#' )
    {
        tokenChar = '\0';
        return tStrConcat;
    }
    else if ( tokenChar.isLetterOrNumber() || ( ExtraAlphaNumChars.find( tokenChar ) >= 0 ) )
    {
        tokenBuffer = tokenChar;
        *stream >> tokenChar;
        while ( !stream->atEnd() && ( tokenChar.isLetterOrNumber() || ( ExtraAlphaNumChars.find( tokenChar ) >= 0 ) ) )
        {
            tokenBuffer += tokenChar;
            *stream >> tokenChar;
        }

        return tAlphaNumText;
    }

    if ( stream->atEnd() )
        return tEOF;

    qDebug( "Unexpected token char '%c' (asc %i) at stream position %li", tokenChar.latin1(), tokenChar.latin1(), stream->device() ->at() );
    *stream >> tokenChar;

    //     emit parseError( 3 );

    return tUnknown;
}

BibTeXElement *BibTeXFileParser::nextElement()
{
    BibTeXElement * result = NULL;

    while ( token != tEOF && token != tUnknown && token != tComment && token != tPreamble && token != tString && token != tEntry && !cancelFlag )
        token = readToken( );

    if ( !cancelFlag )
    {
        switch ( token )
        {
        case tComment:
            result = new BibTeXComment( tokenBuffer );
            break;

        case tPreamble:
            tokenBuffer = readChars( );
            result = new BibTeXPreamble( tokenBuffer ) ;
            break;

        case tString:
            result = readString( );
            break;

        case tEntry:
            {
                QString type = tokenBuffer;
                result = readEntry( type );
            }
            break;
        default:
            result = NULL;
        }

        token = readToken( );
        // for some reason, IEEE .bib files have semicolon after bibtex entries...
        //          if ( token == tSemicolon )
        //              token = readToken( );

        emit progress( stream->device() ->at(), stream->device() ->size() );
        qApp->processEvents();
    }

    return result;
}

void BibTeXFileParser::cancel()
{
    cancelFlag = true;
}

#include "bibtexfileparser.moc"
