/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <string>

#include <qfile.h>
#include <qstring.h>
#include <qtextstream.h>
#include <qcombobox.h>
#include <qprogressdialog.h>
#include <qclipboard.h>
#include <kinstance.h>
#include <kaction.h>
#include <kstdaction.h>
#include <klistview.h>
#include <klocale.h>
#include <kio/netaccess.h>
#include <kfiledialog.h>
#include <ktempfile.h>
#include <kconfigdialog.h>
#include <kmessagebox.h>
#include <kapplication.h>
#include <kstandarddirs.h>
#include <kiconloader.h>
#include <kapplication.h>
#include <kxmlguifactory.h>
#include <kpopupmenu.h>

#include <bibtexcomment.h>
#include <bibtexpreamble.h>
#include <bibtexstring.h>
#include <bibtexentry.h>
#include <bibtexelement.h>
#include <kbibtexlistviewitem.h>
#include <kbibtexentrywidget.h>
#include <kbibtexcommentwidget.h>
#include <kbibtexstringwidget.h>
#include <kbibtexpreamblewidget.h>
#include <kbibtexsettingsdlg.h>
#include <kbibtexactiontextsearch.h>
#include <kbibtexlistview.h>
#include <settings.h>
#include <selectaction.h>
#include "kbibtex_part.h"

Settings kbibtexsettings;

KBibTeXPart::KBibTeXPart( QWidget *parentWidget, const char *widgetName,
                          QObject *parent, const char *name )
        : KParts::ReadWritePart( parent, name ), m_bibtexfile( new BibTeXFile() ), m_newElementCount( 0 ), m_settingsDlg( NULL )
{
    // we need an instance
    setInstance( KBibTeXPartFactory::instance() );

    // set our XML-UI resource file
    setXMLFile( "kbibtex_part.rc" );

    // setup GUI elements
    setupGUI( parentWidget, widgetName );

    // create our actions
    setupActions();

    // we are read-write by default
    setReadWrite( true );

    // we are not modified since we haven't done anything yet
    setModified( false );

    // read configuration
    readSettings();

    updateMenu();
}

KBibTeXPart::~KBibTeXPart()
{
    // save configuration
    writeSettings();

    if ( !m_settingsDlg )
        delete m_settingsDlg;
}

void KBibTeXPart::setReadWrite( bool rw )
{
    ReadWritePart::setReadWrite( rw );
}

void KBibTeXPart::setModified( bool modified )
{
    // get a handle on our Save action and make sure it is valid
    KAction * save = actionCollection() ->action( KStdAction::stdName( KStdAction::Save ) );

    if ( save )
    {
        // if so, we either enable or disable it based on the current
        // state
        save->setEnabled( modified );

        // in any event, we want our parent to do it's thing
        KParts::ReadWritePart::setModified( modified );
    }
}

bool KBibTeXPart::openFile()
{
    bool result = false;

    QFile file( m_file );
    if ( file.open( IO_ReadOnly ) )
    {
        BibTeXFile * new_bibtexfile = new BibTeXFile();

        m_progressDialog = new QProgressDialog( widget() );
        m_progressDialog->setLabelText( QString( i18n( "<qt>Parsing file <b>%1</b></qt>" ) ).arg( m_url.prettyURL() ) );
        connect( new_bibtexfile, SIGNAL( progress( int, int ) ), this, SLOT( showProgress( int, int ) ) );
        connect( this, SIGNAL( progressCancel() ), new_bibtexfile, SLOT( cancel( ) ) );
        connect( m_progressDialog, SIGNAL( canceled() ), new_bibtexfile, SLOT( cancel( ) ) );
        QApplication::setOverrideCursor( Qt::waitCursor );

        result = new_bibtexfile->load( &file );

        disconnect( new_bibtexfile, SIGNAL( progress( int, int ) ), this, SLOT( showProgress( int, int ) ) );
        disconnect( this, SIGNAL( progressCancel() ), new_bibtexfile, SLOT( cancel( ) ) );
        disconnect( m_progressDialog, SIGNAL( canceled() ), new_bibtexfile, SLOT( cancel( ) ) );
        delete m_progressDialog;
        m_progressDialog = NULL;
        QApplication::restoreOverrideCursor();

        file.close();

        if ( result )
        {
            delete m_bibtexfile;
            m_bibtexfile = new_bibtexfile;
            m_bibtexlist->setBibTexFile( m_bibtexfile );
            m_bibtexlist->updateView();

            m_recent->addURL( m_url );

            // just for fun, set the status bar
            emit setStatusBarText( m_url.prettyURL() );

            updateMenu();
        }
        else
            delete new_bibtexfile;

        QApplication::restoreOverrideCursor();
    }

    return result;
}

/**
 * Save to a local file.
 * Uploading will be done by the framework.
 * Defined by KParts::ReadWritePart.
 */
bool KBibTeXPart::saveFile()
{
    // if we aren't read-write, return immediately
    if ( !isReadWrite() )
        return false;

    if ( !url().isValid() )
        return fileSaveAs();
    else
    {
        QString fileName = url().fileName();
        BibTeXFile::Format bibtexFileFormat = BibTeXFile::formatBibTeX;
        if ( fileName.endsWith( ".pdf" ) )
            bibtexFileFormat = BibTeXFile::formatPDF;
        else if ( fileName.endsWith( ".ps" ) )
            bibtexFileFormat = BibTeXFile::formatPS;
        else if ( fileName.endsWith( ".xml" ) )
            bibtexFileFormat = BibTeXFile::formatXML;
        return saveFile( m_file, m_bibtexfile, bibtexFileFormat ) == ssSave;
    }
}

KBibTeXPart::SaveStatus KBibTeXPart::saveFile ( const QString& fileName, BibTeXFile *bibtexfile, BibTeXFile::Format bibtexFileFormat )
{
    SaveStatus result = ssFailed;

    QFile file( fileName );
    qDebug( "filename = %s", fileName.latin1() );
    if ( isReadWrite() && file.open( IO_WriteOnly ) )
    {
        BibTeXFile::Encoding encoding;
        switch ( kbibtexsettings.encoding )
        {
        case Settings::encUTF8:
            encoding = BibTeXFile::encUTF8;
            break;
        default:
            encoding = BibTeXFile::encLaTeX;
        }

        m_progressDialog = new QProgressDialog( widget() );
        m_progressDialog->setLabelText( QString( i18n( "<qt>Saving file <b>%1</b></qt>" ) ).arg( m_url.prettyURL() ) );
        connect( bibtexfile, SIGNAL( progress( int, int ) ), this, SLOT( showProgress( int, int ) ) );
        connect( this, SIGNAL( progressCancel() ), bibtexfile, SLOT( cancel( ) ) );
        connect( m_progressDialog, SIGNAL( canceled() ), bibtexfile, SLOT( cancel( ) ) );
        QApplication::setOverrideCursor( Qt::waitCursor );

        bibtexfile->save( &file, bibtexFileFormat, encoding );
        file.close();

        disconnect( bibtexfile, SIGNAL( progress( int, int ) ), this, SLOT( showProgress( int, int ) ) );
        disconnect( this, SIGNAL( progressCancel() ), bibtexfile, SLOT( cancel( ) ) );
        disconnect( m_progressDialog, SIGNAL( canceled() ), bibtexfile, SLOT( cancel( ) ) );
        delete m_progressDialog;
        m_progressDialog = NULL;
        QApplication::restoreOverrideCursor();

        if ( bibtexFileFormat == BibTeXFile::formatBibTeX )
        {
            emit setStatusBarText( m_url.prettyURL() );
            m_recent->addURL( m_url );
            result = ssSave;
        }
        else
            result = ssExport;
    }

    return result;
}

bool KBibTeXPart::saveFile ( const KURL& url, BibTeXFile *bibtexfile, BibTeXFile::Format bibtexFileFormat )
{
    KTempFile tempfile = KTempFile();
    tempfile.setAutoDelete( TRUE );
    tempfile.close();

    qDebug( "url=%s\n tempfile=%s", url.prettyURL().latin1(), tempfile.name().latin1() );
    return saveFile( tempfile.name(), bibtexfile, bibtexFileFormat ) != ssFailed && KIO::NetAccess::upload( tempfile.name(), url, widget() );
}

bool KBibTeXPart::saveSelectedAs( const KURL& url )
{
    qDebug( "url=%s", url.prettyURL().latin1() );

    QString fileName = url.fileName();
    BibTeXFile::Format bibtexFileFormat = BibTeXFile::formatBibTeX;
    if ( fileName.endsWith( ".pdf" ) )
        bibtexFileFormat = BibTeXFile::formatPDF;
    else if ( fileName.endsWith( ".ps" ) )
        bibtexFileFormat = BibTeXFile::formatPS;
    else if ( fileName.endsWith( ".xml" ) )
        bibtexFileFormat = BibTeXFile::formatXML;

    BibTeXFile * selectedElements = m_bibtexlist->selectedElements();
    bool result = saveFile( url, selectedElements, bibtexFileFormat );
    delete selectedElements;

    return result;
}

void KBibTeXPart::fileOpen()
{
    // this slot is called whenever the File->Open menu is selected,
    // the Open shortcut is pressed (usually CTRL+O) or the Open toolbar
    // button is clicked
    KURL url = KFileDialog::getOpenURL( QString::null, "*.bib|" + i18n( "BibTeX Files (*.bib)" ), widget() );

    if ( url.isValid() )
        openURL( url );
}

bool KBibTeXPart::fileSaveAs()
{
    //     this slot is called whenever the File->Save As menu is selected,
    KURL url = KFileDialog::getSaveURL( QString::null, "*.bib|" + i18n( "BibTeX Files (*.bib)" ) + "\n*.pdf|" + i18n( "Portable Document File (*.pdf)" ) + "\n*.ps|" + i18n( "PostScript (*.ps)" ) + "\n*.xml|" + i18n( "eXtended Markup Language (*.xml)" ), widget() );
    qDebug( "url=%s", url.prettyURL().latin1() );
    if ( url.isValid() && saveAs( url ) )
    {
        setModified( false );
        return true;
    }
    else
        return false;
}

bool KBibTeXPart::fileSaveSelectedAs()
{
    //     this slot is called whenever the File->Save As menu is selected,
    KURL url = KFileDialog::getSaveURL( QString::null, "*.bib|" + i18n( "BibTeX Files (*.bib)" ) + "\n*.pdf|" + i18n( "Portable Document File (*.pdf)" ) + "\n*.ps|" + i18n( "PostScript (*.ps)" ) + "\n*.xml|" + i18n( "eXtended Markup Language (*.xml)" ), widget(), i18n( "Save Selected As" ) );
    qDebug( "url=%s", url.prettyURL().latin1() );
    return url.isValid() && saveSelectedAs( url );
}

void KBibTeXPart::setupGUI( QWidget *parentWidget, const char *name )
{
    // this should be your custom internal widget
    m_bibtexlist = new KBibTeXListView( this, parentWidget, name );
    m_bibtexlist->setBibTexFile( m_bibtexfile );

    // notify the part that this is our internal widget
    setWidget( m_bibtexlist );
}

void KBibTeXPart::setupActions()
{
    KStdAction::open( this, SLOT( fileOpen() ), actionCollection() );
    KStdAction::save( this, SLOT( save() ), actionCollection() );
    KStdAction::saveAs( this, SLOT( fileSaveAs() ), actionCollection() );
    m_actionFileSaveSelectedAs = new KAction( i18n( "Save Selected As..." ), 0, this, SLOT( fileSaveSelectedAs() ), actionCollection(), "file_save_selected_as" );

    m_actionEditCut = KStdAction::cut( this, SLOT( cutElement() ), actionCollection() );
    m_actionEditCopy = KStdAction::copy( this, SLOT( copyElement() ), actionCollection() );
    m_actionEditPaste = KStdAction::paste( this, SLOT( pasteElement() ), actionCollection() );

    m_actionPreferences = KStdAction::preferences( this, SLOT( slotPreferences() ), actionCollection() );

    m_actionEditSearchInternetGoogle = new KAction( i18n( "Search &Google" ), 0, this, SLOT( slotSearchGoogle() ), actionCollection(), "edit_searchinternet_google" );
    m_actionEditSearchInternetGoogleScholar = new KAction( i18n( "Search Google &Scholar" ), 0, this, SLOT( slotSearchGoogleScholar() ), actionCollection(), "edit_searchinternet_googlescholar" );
    m_actionEditSearchInternetCiteseer = new KAction( i18n( "Search &CiteSeer" ), 0, this, SLOT( slotSearchCiteseer() ), actionCollection(), "edit_searchinternet_citeseer" );
    m_actionEditSearchInternetCiteseer = new KAction( i18n( "Search &CiteSeer" ), 0, this, SLOT( slotSearchCiteseer() ), actionCollection(), "edit_searchinternet_citeseer" );

    m_actionElementsList = new KBibTeX::SelectAction( i18n( "Add Elements" ), 0, actionCollection(), "elements_list" );
    QPtrList<KAction> alist;
    alist.append( new KAction( i18n( "Article" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_article" ) );
    alist.append( new KAction( i18n( "Book" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_book" ) );
    alist.append( new KAction( i18n( "Booklet" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_booklet" ) );
    alist.append( new KAction( i18n( "Electronic (IEEE)" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_electronic" ) );
    alist.append( new KAction( i18n( "InBook" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_inbook" ) );
    alist.append( new KAction( i18n( "InCollection" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_incollection" ) );
    alist.append( new KAction( i18n( "InProceedings" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_inproceedings" ) );
    alist.append( new KAction( i18n( "Manual" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_manual" ) );
    alist.append( new KAction( i18n( "MastersThesis" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_mastersthesis" ) );
    alist.append( new KAction( i18n( "Misc" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_misc" ) );
    alist.append( new KAction( i18n( "PhDThesis" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_phdthesis" ) );
    alist.append( new KAction( i18n( "Proceedings" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_proceedings" ) );
    alist.append( new KAction( i18n( "TechReport" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_techreport" ) );
    alist.append( new KAction( i18n( "Unpublished" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_entry_unpublished" ) );
    alist.append( new KAction( i18n( "Comment" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_comment" ) );
    alist.append( new KAction( i18n( "Preamble" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_preamble" ) );
    alist.append( new KAction( i18n( "String" ), 0, this, SLOT( slotNewElement() ), actionCollection(), "element_new_string" ) );
    m_actionElementsList->setItems( alist );

    m_actionEditElement = new KAction( i18n( "&Edit" ), 0, this, SLOT( editElement() ), actionCollection(), "element_edit" );
    m_actionDeleteElement = new KAction( i18n( "&Delete" ), 0, this, SLOT( deleteElement() ), actionCollection(), "element_delete" );

    m_recent = new KRecentFilesAction( i18n( "Open &Recent Files" ), 0,
                                       this, SLOT( openURL( const KURL & ) ),
                                       actionCollection(), "file_open_recent" );

    KBibTeXActionTextSearch *actionTextSearch = new KBibTeXActionTextSearch( i18n( "Search" ), 0, actionCollection(), "search_text" );
    new KAction( i18n( "Clear search text" ), QApplication::reverseLayout() ? "clear_left" : "locationbar_erase", CTRL + Key_L, actionTextSearch, SLOT( clear() ), actionCollection(), "clear_search_text" );

    connect( m_bibtexlist, SIGNAL( doubleClicked( QListViewItem*, const QPoint&, int ) ), this, SLOT( editElement( QListViewItem* ) ) );
    // connect( m_bibtexlist, SIGNAL( <futed( QListViewItem* ) ), this, SLOT( slotEditElement( QListViewItem* ) ) );
    connect( m_bibtexlist, SIGNAL( selectionChanged() ), this, SLOT( slotSelectionChanged( ) ) );
    connect( m_bibtexlist, SIGNAL( contextMenu( KListView *, QListViewItem *, const QPoint & ) ), this, SLOT( showBibtexListContextMenu( KListView *, QListViewItem *, const QPoint & ) ) );
    connect( actionTextSearch, SIGNAL( doSearch( const QString& ) ), m_bibtexlist, SLOT( filterText( const QString& ) ) );
}

void KBibTeXPart::slotNewElement()
{
    BibTeXElement * newElement = NULL;

    if ( isReadWrite() && sender() )
    {
        QString newId = QString( i18n( "NewElement_%1" ) ).arg( m_newElementCount++ );

        if ( strncmp( sender() ->name(), "element_new_entry_", 18 ) == 0 )
        {
            BibTeXEntry::EntryType entryType = BibTeXEntry::entryTypeFromString( sender() ->name() + 18 );

            if ( entryType != BibTeXEntry::etUnknown )
                newElement = new BibTeXEntry( entryType, newId );
        }
        else if ( strcmp( sender() ->name(), "element_new_comment" ) == 0 )
        {
            newElement = new BibTeXComment( i18n( "Empty comment" ) );
        }
        else if ( strcmp( sender() ->name(), "element_new_preamble" ) == 0 )
        {
            newElement = new BibTeXPreamble( i18n( "Empty preamble" ) );
        }
        else if ( strcmp( sender() ->name(), "element_new_string" ) == 0 )
        {
            newElement = new BibTeXString( newId, i18n( "Empty string" ) );
        }

        if ( newElement != NULL && editElement( newElement ) )
        {
            m_bibtexfile->appendElement( newElement );
            m_bibtexlist->updateView();
        }
        else
            delete newElement;
    }
}

bool KBibTeXPart::editElement( QListViewItem * item )
{
    bool result = false;

    if ( item == NULL )
        item = m_bibtexlist->selectedItem();
    if ( item == NULL )
        item = m_bibtexlist->currentItem();
    if ( item == NULL )
        return false;

    KBibTeXListViewItem * kblvi = dynamic_cast<KBibTeXListViewItem*>( item );
    if ( kblvi != NULL )
        result = editElement( kblvi->element() );
    if ( result )
        m_bibtexlist->updateView( kblvi );

    return result;
}

bool KBibTeXPart::editElement( BibTeXElement* element )
{
    QDialog::DialogCode dialogResult = QDialog::Rejected;

    BibTeXEntry * entry = dynamic_cast<BibTeXEntry*>( element );
    if ( entry )
        dialogResult = KBibTeXEntryWidget::execute( entry, m_bibtexfile, i18n( "Edit BibTeX Entry" ), isReadWrite() );
    else
    {
        BibTeXComment* comment = dynamic_cast<BibTeXComment*>( element );
        if ( comment )
            dialogResult = KBibTeXCommentWidget::execute( comment, i18n( "Edit BibTeX Comment" ), isReadWrite() );
        else
        {
            BibTeXString* string = dynamic_cast<BibTeXString*>( element );
            if ( string )
                dialogResult = KBibTeXStringWidget::execute( string, i18n( "Edit BibTeX String" ), isReadWrite() );
            else
            {
                BibTeXPreamble* preamble = dynamic_cast<BibTeXPreamble*>( element );
                if ( preamble )
                    dialogResult = KBibTeXPreambleWidget::execute( preamble, i18n( "Edit BibTeX Preamble" ), isReadWrite() );
            }
        }
    }

    if ( dialogResult == QDialog::Accepted )
        setModified( true );

    return dialogResult == QDialog::Accepted;
}

void KBibTeXPart::deleteElement()
{
    if ( isReadWrite() )
    {
        QListViewItemIterator it( m_bibtexlist, QListViewItemIterator::Selected );
        while ( it.current() )
        {
            KBibTeXListViewItem * kblvi = dynamic_cast<KBibTeXListViewItem*>( it.current() );
            m_bibtexfile->deleteElement( kblvi->element() );
            it++;
        }

        m_bibtexlist->updateView();
        setModified( true );
    }
}

void KBibTeXPart::showProgress( int current, int total )
{
    if ( m_progressDialog != NULL )
    {
        m_progressDialog->setTotalSteps( total );
        m_progressDialog->setProgress( current );
        if ( m_progressDialog->wasCanceled() )
            emit progressCancel();
    }

    kapp->processEvents();
}

void KBibTeXPart::readSettings()
{
    KConfig * config = KBibTeXPartFactory::instance() ->config();

    config->setGroup( "Session" );
    m_recent->loadEntries( config );

    kbibtexsettings.load( config );
    m_bibtexlist->setSorting( kbibtexsettings.sortingColumn );
}

void KBibTeXPart::writeSettings()
{
    KConfig * config = KBibTeXPartFactory::instance() ->config();

    config->setGroup( "Session" );
    m_recent->saveEntries( config );

    kbibtexsettings.save( config );
}

void KBibTeXPart::cutElement()
{
    copyElement();
    if ( isReadWrite() )
        deleteElement();
}

void KBibTeXPart::copyElement()
{
    QString buffer;
    QListViewItemIterator it( m_bibtexlist, QListViewItemIterator::Selected );
    while ( it.current() )
    {
        KBibTeXListViewItem * kblvi = dynamic_cast<KBibTeXListViewItem*>( it.current() );

        buffer.append( kblvi->element() ->toString() );
        buffer.append( "\n" );

        it++;
    }

    kapp->clipboard() ->setText( buffer );
}

void KBibTeXPart::pasteElement()
{
    QString clipText( kapp->clipboard() ->text() );
    insertText( clipText );
}

void KBibTeXPart::showBibtexListContextMenu( KListView *, QListViewItem *, const QPoint & p )
{
    if ( factory() )
    {
        updateMenu();
        KPopupMenu* popupMenu = static_cast<KPopupMenu*>( factory() ->container( "popup_bibtexlist", this ) );
        popupMenu ->popup( p );
    }
}

void KBibTeXPart::updateMenu()
{
    int numSelected = 0;
    QListViewItemIterator it( m_bibtexlist, QListViewItemIterator::Selected );
    while ( it.current() )
    {
        numSelected++;
        it++;
    }

    m_actionFileSaveSelectedAs->setEnabled( numSelected > 0 );
    m_actionElementsList->setEnabled( isReadWrite() );
    m_actionEditElement->setEnabled( numSelected == 1 );
    m_actionDeleteElement->setEnabled( isReadWrite() && numSelected > 0 );
    m_actionEditCut->setEnabled( isReadWrite() && numSelected > 0 );
    m_actionEditCopy->setEnabled( numSelected > 0 );
    m_actionEditSearchInternetGoogle->setEnabled( numSelected == 1 );
    m_actionEditSearchInternetGoogleScholar->setEnabled( numSelected == 1 );
    m_actionEditSearchInternetCiteseer->setEnabled( numSelected == 1 );
}

void KBibTeXPart::slotSelectionChanged()
{
    updateMenu();
}

void KBibTeXPart::insertText( QString & text )
{
    if ( isReadWrite() )
    {
        m_bibtexfile->insertText( text );
        m_bibtexlist->updateView();

        setModified( true );
    }
}

void KBibTeXPart::slotSearchGoogle()
{
    QListViewItemIterator it( m_bibtexlist, QListViewItemIterator::Selected );
    KBibTeXListViewItem * kblvi = dynamic_cast<KBibTeXListViewItem*>( it.current() );
    if ( kblvi != NULL )
        searchInternet( kblvi->element(), iseGoogle );
}

void KBibTeXPart::slotSearchGoogleScholar()
{
    QListViewItemIterator it( m_bibtexlist, QListViewItemIterator::Selected );
    KBibTeXListViewItem * kblvi = dynamic_cast<KBibTeXListViewItem*>( it.current() );
    if ( kblvi != NULL )
        searchInternet( kblvi->element(), iseGoogleScholar );
}

void KBibTeXPart::slotSearchCiteseer()
{
    QListViewItemIterator it( m_bibtexlist, QListViewItemIterator::Selected );
    KBibTeXListViewItem * kblvi = dynamic_cast<KBibTeXListViewItem*>( it.current() );
    if ( kblvi != NULL )
        searchInternet( kblvi->element(), iseCiteSeer );
}

void KBibTeXPart::searchInternet( BibTeXElement * element, InternetSearchEngine engine )
{
    QString queryString = QString::null;

    BibTeXEntry* entry = dynamic_cast<BibTeXEntry*>( element );
    if ( entry != NULL )
        queryString = entry->getField( BibTeXEntryField::ftTitle ) ->value();
    else
    {
        BibTeXComment* comment = dynamic_cast<BibTeXComment*>( element );
        if ( comment != NULL )
            queryString = comment->comment();
        else
        {
            BibTeXString* string = dynamic_cast<BibTeXString*>( element );
            if ( string != NULL )
                queryString = string->value();
            else
                qDebug( "Not yet supported" );
        }
    }

    if ( queryString != QString::null )
        switch ( engine )
        {
        case iseGoogle: kapp->invokeBrowser( QString( "http://www.google.com/search?q=%1&ie=UTF-8&oe=UTF-8" ).arg( queryString ) );
            break;
        case iseGoogleScholar: kapp->invokeBrowser( QString( "http://scholar.google.com/scholar?q=%1&ie=UTF-8&oe=UTF-8" ).arg( queryString ) );
            break;
        case iseCiteSeer: kapp->invokeBrowser( QString( "http://sherry.ifi.unizh.ch/cs?q=%1&submit=Search+Documents&cs=1" ).arg( queryString ) );
            break;
        }
}

void KBibTeXPart::slotPreferences()
{
    if ( !m_settingsDlg )
        m_settingsDlg = new KBibTeXSettingsDlg( widget(), "settings_dlg" );

    if ( m_settingsDlg->exec() == QDialog::Accepted )
    {
        m_bibtexlist->setSorting( kbibtexsettings.sortingColumn );
    }
}

// It's usually safe to leave the factory code alone.. with the
// notable exception of the KAboutData data
#include <kaboutdata.h>
#include <klocale.h>

KInstance* KBibTeXPartFactory::s_instance = 0L;
KAboutData* KBibTeXPartFactory::s_about = 0L;

KBibTeXPartFactory::KBibTeXPartFactory()
        : KParts::Factory()
{}

KBibTeXPartFactory::~KBibTeXPartFactory()
{
    delete s_instance;
    delete s_about;

    s_instance = 0L;
}

KParts::Part* KBibTeXPartFactory::createPartObject( QWidget *parentWidget, const char *widgetName,
        QObject *parent, const char *name,
        const char *classname, const QStringList & /* args */ )
{
    // Create an instance of our Part
    KBibTeXPart * obj = new KBibTeXPart( parentWidget, widgetName, parent, name );

    // See if we are to be read-write or not
    if ( QCString( classname ) == "KParts::ReadOnlyPart" )
        obj->setReadWrite( false );

    return obj;
}

KInstance* KBibTeXPartFactory::instance()
{
    if ( !s_instance )
    {
        s_about = new KAboutData( "kbibtexpart", I18N_NOOP( "KBibTeXPart" ), "0.1.1" );
        s_about->addAuthor( "Thomas Fischer", 0, "fischer@unix-ag.uni-kl.de" );
        s_instance = new KInstance( s_about );
    }
    return s_instance;
}

extern "C"
{
    void* init_libkbibtexpart()
    {
        return new KBibTeXPartFactory;
    }
};

#include "kbibtex_part.moc"

